/**********************************************************************/
/*      Licensed Materials - Property of IBM                          */
/*      "Restricted Materials of IBM"                                 */
/*      5748-T15                                                      */
/*      Copyright IBM Corp. 2016                                      */
/*      All Rights Reserved                                           */
/*      US Government Users Restricted Rights - Use, duplication      */
/*      or disclosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                                */
/**********************************************************************/
/*                                                                    */
/* FUNCTION NAME.....QHSC                                             */
/* PURPOSE...........QHSC driver                                      */
/* FUNCTION TYPE.....TPF                                              */
/*                                                                    */
/* INPUT.......ZTEST CONN                                             */
/*                                                                    */
/* Options:                                                           */
/*                                                                    */
/* DESCRIPTION: QHSC is the program that tests the high               */
/*              speed connector API function                          */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DEPENDENCIES... None.                                              */
/*                                                                    */
/* RESTRICTIONS... None.                                              */
/*                                                                    */
/* SPECIAL CONSIDERATIONS...                                          */
/*                                                                    */
/* INTERFACES:                                                        */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* CHANGE ACTIVITY...                                                 */
/*                                                                    */
/*   PLOG       DATE       DESCRIPTION                                */
/*   ----       ----       -----------                                */
/* f26077       03/28/2016 Create High Speed Connector driver         */
/**********************************************************************/

#include <tpf/connapi.h>
#include <tpf/tpfparse.h>                                                         
#include <tpf/tpfapi.h>                                                         
#include <string>                                                               

using namespace std;                                                           

/* DRIVER ERROR MESSAGES */
#define QHSC0102TXT     "CONN ERROR, NUMBER OF REQUESTS MUST BE GREATER THAN 0"
#define QHSC0103TXT     "CONN ERROR, READ MUST BE Y OR N"
#define QHSC0104TXT     "CONN ERROR, RESPONSE SIZE MUST BE GREATER THAN OR EQUAL TO 32"
#define QHSC0105TXT     "CONN ERROR, REQUEST SIZE MUST BE BETWEEN 1 and 1048576"
#define QHSC0106TXT     "CONN ERROR, TIMEOUT MUST BE BETWEEN 1 AND 600000"
#define QHSC0107TXT     "CONN ERROR, RESP_SIZE REQUIRED IF READ=Y"
#define QHSC0108TXT     "CONN ERROR, RRobin MUST BE Y OR N"
#define QHSC0109TXT     "CONN ERROR, SYSBUFF MUST BE Y OR N"
                               
/*API ERRORS*/
#define MSGAPI000TXT    "TPF_SEND_MESSAGE() API SUCCESSFUL"
#define MSGAPI001TXT    "TPF_SEND_MESSAGE() API ERROR: INVALID ENDPOINT GROUP NAME"
#define MSGAPI002TXT    "TPF_SEND_MESSAGE() API ERROR: NO REQUEST BUFFER PROVIDED"
#define MSGAPI003TXT    "TPF_SEND_MESSAGE() API ERROR: INVALID TIMEOUT PARAMETER"
#define MSGAPI004TXT    "TPF_SEND_MESSAGE() API ERROR: REQUEST SIZE TOO SMALL"
#define MSGAPI005TXT    "TPF_SEND_MESSAGE() API ERROR: RESPONSE BUFFER TOO SMALL"
#define MSGAPI006TXT    "TPF_SEND_MESSAGE() API ERROR: TPF_SEND_MESSAGE() TIMED OUT ON REQUEST"
#define MSGAPI007TXT    "TPF_SEND_MESSAGE() API ERROR: A SOCKET ERROR OCCURED DURING THE PROCESSING OF THIS FUNCTION, CAUSING IT TO BE CLOSED"
#define MSGAPI008TXT    "TPF_SEND_MESSAGE() API ERROR: THE RESPONSE MESSAGE IS TOO SMALL"
#define MSGAPI009TXT    "TPF_SEND_MESSAGE() API ERROR: NO SESSION IS AVAILABLE"

/* HELP MESSAGES */
#define QHSCHELP0000    "THIS IS THE SOCKET CLIENT DRIVER APPLICATION FOR TPF."
#define QHSCHELP0001    "THE PURPOSE OF THIS DRIVER IS TO TEST THE tpf_send_message() API ON TPF."
#define QHSCHELP0002    "DRIVER COMMANDS:"
#define QHSCHELP0003    "ZTEST CONN ENDGroup-u+++++++ Read-a REQuest-d++++++++ REQSize-d++++++ [RESPSize-d++++] [RRobin-a] Timeout-d++++++ SYSBUFF-a"
#define QHSCHELP0004    "REQUIRED PARAMETERS DESCRIPTION: "
#define QHSCHELP0005    "   ENDGroup-THE 8 ALPHANUMERIC CHARACTER NAME OF THE ENDPOINT GROUP TO SEND MESSAGES TO"
#define QHSCHELP0006    "       Read-[Y/N], SHOULD THE CLIENT LISTEN FOR A RESPONSE (IF Y, RESPSize MUST BE SPECIFIED)"
#define QHSCHELP0007    "    REQuest-[1-999999999] NUMBER OF REQUESTS TO SEND"
#define QHSCHELP0008    "    REQSize-NUMBER OF BYTES EACH REQUEST WILL BE"
#define QHSCHELP0009    "    Timeout-[1-600000] MILLISECONDS TO TIMEOUT"
#define QHSCHELP0013    "    Sysbuff-[Y/N], IF RESPONSE REQUESTED, SPECIFY Y IF SYSTEM WILL PROVIDE RESPONSE BUFFER, N IF USER"
#define QHSCHELP0010    "OPTIONAL PARAMETERS DESCRIPTION: "
#define QHSCHELP0011    "   RESPSize- NUMBER OF BYTES OF THE RESPONSE MESSAGE"
#define QHSCHELP0012    "     RRobin-[Y/N] REQUEST THAT THE SERVER USES ROUND ROBIN WHEN USING ENDPOINTS TO RESPOND"

#define COMMAND_CODE "QHSC"  //Message prefix
#define DRVR_MSG_HEADER 1    //used with WTOPC wrapper function conn_drvr_message

/* Send conn driver message to console. */
void conn_drvr_message(char *msgbuf, enum t_wtopc_chain msg_chain, char msg_header,   
                       int msgnum, char msglet, char *msg_prefix);
																							
/* Return code error handler */
void tpf_send_message_err_chk(int rc, char* message_buffer);

/* Help screen */
void CONN_help();
void CONN_help()
{

    char message_buffer[400];
    
    sprintf(message_buffer,QHSCHELP0000);
    conn_drvr_message(message_buffer,
                      WTOPC_NO_CHAIN,
                      DRVR_MSG_HEADER,
                      000,
                      'I',
                      COMMAND_CODE);
                      
    sprintf(message_buffer,QHSCHELP0001);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);
                      
    sprintf(message_buffer,QHSCHELP0002);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);
                      
    sprintf(message_buffer,QHSCHELP0003);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);
                      
    sprintf(message_buffer,QHSCHELP0004);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);

    sprintf(message_buffer,QHSCHELP0005);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);

    sprintf(message_buffer,QHSCHELP0006);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);

    sprintf(message_buffer,QHSCHELP0007);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);

    sprintf(message_buffer,QHSCHELP0008);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);      
                      
    sprintf(message_buffer,QHSCHELP0009);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);

    sprintf(message_buffer,QHSCHELP0013);
      conn_drvr_message(message_buffer,
                        WTOPC_CHAIN,
                        0,
                        0,
                        ' ',
                        COMMAND_CODE);
                      
    sprintf(message_buffer,QHSCHELP0010);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);    
                      
    sprintf(message_buffer,QHSCHELP0011);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);       
                      
    sprintf(message_buffer,QHSCHELP0012);
    conn_drvr_message(message_buffer,
                      WTOPC_CHAIN_END,
                      0,
                      0,
                      ' ',
                      COMMAND_CODE);                      

}

/*****************************************/
/* Grammar used for parsing command line */
/*****************************************/
char *grammar = \
    "{"

         "{"

         "ENDGroup-u+++++++ Read-a REQuest-d++++++++ REQSize-d++++++ [RESPSize-d++++++] [RRobin-a] Timeout-d++++++ SYSBUFF-a"

         "}"

     "}";

	
/***********************/
/* QHSC main procedure */
/***********************/
extern "C" void QHSC() {

  /**********************************************************************/
  /*       Variables needed to parse the input message                  */
  /**********************************************************************/
  int                 num_parms=0;        /* IPRSE return code          */
  struct IPRSE_output parse_results;      /* Structure for parsed input */
  struct IPRSE_output *parse_ptr;         /* Pointer to current parm    */
  char                *input_ptr;         /* Pointer to command block   */
  char                *msg_ptr;

  int      message_type=0;    // Set the message type
  int      read_opt=0;        // read Y/N
  int      sysbuff_opt = 0;   // system allocates buffer = 1, otherwise 0    
  int      rr_opt=1;          // Option for round robin, will use if 1, 0 means user chooses ept
  char     *read_ptr = NULL, *rr_ptr = NULL, *sysbuff_ptr = NULL; // Used for parsing input
                                                                  //   for read_opt, sysbuff_opt, rr_opt
  int      request=0;         // Number of requests
  int      req_size=0;        // size of request message, not including msg type.
  int      msg_size=0;        // size of message, which includes msg type.
  int      resp_size=0;       // optional, size of each response buffer
  int      timeout=0;         // timeout value
  int      rc, l;
  char     *request_buffer = NULL, *response_buffer = NULL;
  string   endgroup;
  char     *endpoint_group = NULL;
  char      message_buffer[400];
  char      endpoint_in[9], endpoint_out[9];
  hsc       connector_parms;   //declare connector_parms in stck
  struct    hsc_req *req_buffer;

/**********************************************************************/
/* Parse parameters from ZTEST CONN command.                          */
/**********************************************************************/
  // Start of input  command
  input_ptr = (char *) ecbp2()->ce2crsmsg;

  /* If input contains '_', replace '_' with '&' to avoid parser error */
  /* If input contains '/', replace '/' with '$' to avoid parser error */
  if ((input_ptr = strstr(input_ptr, "_"))||(input_ptr = strstr(input_ptr, "/")))
  {  if ((input_ptr = strstr(input_ptr, "_")))
         *input_ptr++ = '&';
     else
         *input_ptr++ = '$';
     while ((input_ptr = strstr(input_ptr, "_"))||(input_ptr = strstr(input_ptr, "/")))  {
        if ((input_ptr = strstr(input_ptr, "_")))
           *input_ptr++ = '&';
        else
           *input_ptr++ = '$';
     }
  }

  input_ptr = (char *) ecbp2()->ce2crsmsg;
    msg_ptr = input_ptr + strlen("ztest") + strlen(COMMAND_CODE) + 1;

    num_parms = IPRSE_parse(msg_ptr,
                            grammar,
                            &parse_results,
                            IPRSE_PRINT + IPRSE_ALLOC + IPRSE_MIXED_CASE + IPRSE_STRICT,
                            COMMAND_CODE);

/**********************************************************************/
/*  If '?' or HELP was input, display help menu.                      */
/**********************************************************************/
    if (num_parms == IPRSE_HELP) {
       CONN_help();
       exit(0);
       }

/**********************************************************************/
/* If error in command syntax, exit (IPRSE_parse already issued msg)  */
/**********************************************************************/
    if (num_parms == 0)
       exit(0);

/**********************************************************************/
/*  If message gets bad return code from IPRSE, then exit.            */
/**********************************************************************/
    if (num_parms == IPRSE_BAD)  {
       exit(0);
       }

    parse_ptr = &parse_results;

    /******************************************************************/
    /* Loop through all CONN parameters                               */
    /******************************************************************/
    int i;
    for (i = 1, parse_ptr = &parse_results;
                i <= num_parms;
                i++, parse_ptr= parse_ptr->IPRSE_next) {

          /****************************************************************/
          /* Parse CONN parameters - ENDGROUP parameter                   */
          /****************************************************************/
          if (strncmp(parse_ptr->IPRSE_parameter,"ENDGroup-",9) == 0)
          {
             endgroup = parse_ptr->IPRSE_value;
             endpoint_group = (char *) endgroup.c_str();
          }
          /****************************************************************/
          /* Parse CONN parameters - READ parameter                       */
          /****************************************************************/
          else if (strncmp(parse_ptr->IPRSE_parameter,"Read-",5) == 0)
             read_ptr = parse_ptr->IPRSE_value;
          /****************************************************************/
          /* Parse CONN parameters - SYSBUFF parameter                    */
          /****************************************************************/     
          else if (strncmp(parse_ptr->IPRSE_parameter,"SYSBUFF-",8) == 0)       
             sysbuff_ptr = parse_ptr->IPRSE_value;
          /****************************************************************/
          /* Parse CONN parameters - ROUND ROBIN parameter                */
          /****************************************************************/
          else if (strncmp(parse_ptr->IPRSE_parameter,"RRobin-",7) == 0)
             rr_ptr = parse_ptr->IPRSE_value;
          /****************************************************************/
          /* Parse CONN parameters - REQUEST parameter                    */
          /****************************************************************/
          else if (strncmp(parse_ptr->IPRSE_parameter,"REQuest-",8) == 0)
             request = atoi(parse_ptr->IPRSE_value);
          /****************************************************************/
          /* Parse CONN parameters - REQUEST SIZE parameter               */
          /****************************************************************/
          else if (strncmp(parse_ptr->IPRSE_parameter,"REQSize-",8) == 0)
          {
             req_size = atoi(parse_ptr->IPRSE_value);
             if (req_size < 0)
             {
                 printf("request size is not valid");
                 exit(0);
             }
          }
          /****************************************************************/
          /* Parse CONN parameters - RESPONSE SIZE parameter              */
          /****************************************************************/
          else if (strncmp(parse_ptr->IPRSE_parameter,"RESPSize-",9) == 0)
          {
             resp_size = atoi(parse_ptr->IPRSE_value);
          }
          /****************************************************************/
          /* Parse CONN parameters - TIMEOUT parameter                    */
          /****************************************************************/
          else if (strncmp(parse_ptr->IPRSE_parameter,"Timeout-",8) == 0)  
             timeout = atoi(parse_ptr->IPRSE_value); 

   } /*end while*/
   /* Number of requests must be greater than zero */
   if(request<=0) 
   {
       sprintf(message_buffer,QHSC0102TXT);
       conn_drvr_message(message_buffer,
                         WTOPC_NO_CHAIN,
                         DRVR_MSG_HEADER,
                         102,
                         'E',
                         COMMAND_CODE);
       exit(0);
   }
   /* Request size must be greater than 1 */
   if(req_size < 1)
   {
      sprintf(message_buffer,QHSC0105TXT);
      conn_drvr_message(message_buffer,
                        WTOPC_NO_CHAIN,
                        DRVR_MSG_HEADER,
                        105,
                        'E',
                        COMMAND_CODE);
      exit(0);
   }
   /* verify timeout value is valid */
   if(timeout<=0 || timeout > 600000)
   {
      sprintf(message_buffer,QHSC0106TXT);
      conn_drvr_message(message_buffer,
                        WTOPC_NO_CHAIN,
                        DRVR_MSG_HEADER,
                        106,
                        'E',
                        COMMAND_CODE);
      exit(0); 
   }
			/* Verify round robin parameter is valid */
   if (rr_ptr != NULL)
   {
       if(*rr_ptr == 'Y' || *rr_ptr == 'y')
           rr_opt = 1;
       else if(*rr_ptr == 'N' || *rr_ptr == 'n')
           rr_opt = 0;
       else 
       {
       sprintf(message_buffer,QHSC0108TXT);
       conn_drvr_message(message_buffer,
                         WTOPC_NO_CHAIN,
                         DRVR_MSG_HEADER,
                         108,
                         'E',
                         COMMAND_CODE);
       exit(0);
       }
   }
			/* Verify sysbuff parameter is valid */
   if(*sysbuff_ptr == 'Y' || *sysbuff_ptr == 'y')
        sysbuff_opt = 1;
   else if(*sysbuff_ptr == 'N' || *sysbuff_ptr == 'n')
        sysbuff_opt = 0;
   else 
   {   
        sprintf(message_buffer, QHSC0109TXT);
        conn_drvr_message(message_buffer,                                                
                         WTOPC_NO_CHAIN,                                                 
                         DRVR_MSG_HEADER,                                                
                         109,                                                            
                         'E',                                                            
                         COMMAND_CODE);                                                
       exit(0);                                                                                 

   }
			/* Verify read parameter is valid */
   if(*read_ptr == 'Y' || *read_ptr == 'y')
   {
       read_opt = 1;
       message_type = 100;
       if(resp_size==0)  //If we are reading, we must supply response buffer size
       {
         sprintf(message_buffer,QHSC0107TXT);
         conn_drvr_message(message_buffer,
                           WTOPC_NO_CHAIN,
                           DRVR_MSG_HEADER,
                           107,
                           'E',
                           COMMAND_CODE);
	 exit(0);
       }
       if(resp_size < HSC_HEADER_SIZE)
       {
         sprintf(message_buffer,QHSC0104TXT);
         conn_drvr_message(message_buffer,
                           WTOPC_NO_CHAIN,
                           DRVR_MSG_HEADER,
                           104,
                           'E',
                           COMMAND_CODE);
         exit(0);
       }
       if(sysbuff_opt == 0)
           response_buffer = (char *) malloc(resp_size);
   }
   else if(*read_ptr == 'N' || *read_ptr == 'n')
   {
       read_opt = 0;
       message_type = 101;
   }
   else 
   {
      sprintf(message_buffer,QHSC0103TXT);
      conn_drvr_message(message_buffer,
                        WTOPC_NO_CHAIN,
                        DRVR_MSG_HEADER,
                        103,
                        'E',
                        COMMAND_CODE);
      exit(0);
   }
	  
   request_buffer = (char *) malloc(req_size+HSC_HEADER_SIZE);          // Initialize storage for request_buffer
   req_buffer = (struct hsc_req *) request_buffer;
   memset(request_buffer, 0, HSC_HEADER_SIZE);                          //zero out header
   /* Message size does not include byte count field at beginning of buffer. */
   msg_size = req_size + (HSC_HEADER_SIZE - sizeof(req_buffer->length));
   req_buffer->length = msg_size;               // Fill first 4 bytes of request_buffer with message length.
   req_buffer->header_length = HSC_HEADER_SIZE; //Fill next 2 bytes with header size.
   req_buffer->type = message_type;             // Fill next 2 bytes with the message type
   memset(req_buffer->data, 0xC1, req_size);    // Fill message in with EBCDIC char 'AAAAA...'.

   //Initialize our HSC structure
   connector_parms.version       = HSC_VERSION_1;   //version 1
   connector_parms.endpointGroup = endpoint_group;  //name of the endpoint group to send to
   connector_parms.request       = request_buffer;  //buffer for the request message
   connector_parms.timeout       = timeout;         //API timeout value
   connector_parms.response      = response_buffer; //Pointer to response buffer, either allocated by the client or the server.
   connector_parms.resp_len      = resp_size;       //Size of the response message, if no response requested, value is 0.
                                                    //If server provides the buffer, resp_size is the max allowed response size
                                                    
   /* If round robin not specified, then pass address where endpoint_out will be copied to. */
   if (rr_opt == 0)
      connector_parms.endpoint_in   = endpoint_in;  //Send all requests to the same endpoint
   else
      connector_parms.endpoint_in   = NULL;         //Round robin requests
   
   connector_parms.endpoint_out  = endpoint_out;
			
   if(rr_opt == 0) 
   {
       rc = tpf_send_message(&connector_parms);  //Issue first API
       if(rc!=TPF_SEND_MESSAGE_OK)               //if API was not successful, process error code
	          tpf_send_message_err_chk(rc, message_buffer);
       if(request>=2)                            //save us from copying if request==1
           memcpy(endpoint_in, endpoint_out, sizeof(endpoint_in)); //Use endpoint passed back from API
       request--; //this is a request, so decrement by one
			}
   
   for(l=0; l<request; l++)                      //Issue the rest of our requests
   {
       rc = tpf_send_message(&connector_parms);  //Issue API
       if(rc!=TPF_SEND_MESSAGE_OK)               //If error, handle error code
	          tpf_send_message_err_chk(rc, message_buffer);
           
       /* The following code sleeps for 50 milliseconds after 100000 messages to prevent a CTL-10. */
       /* When there is no response from the server, we continue to issue tpf_send_message() in a loop without losing control. */ 
       /*     Sleep to give up control. */
       if ((l % 100 == 0) && (read_opt == 0))
          dlayc();
   }
   
   //All of our APIs were successful
   sprintf(message_buffer,MSGAPI000TXT);
   conn_drvr_message(message_buffer,
                     WTOPC_NO_CHAIN,
                     DRVR_MSG_HEADER,
                     000,
                     'I',
                     COMMAND_CODE);   
   exit(0);                                                                                

}

/* Prints the appropriate error message and exit */
void tpf_send_message_err_chk(int rc, char* message_buffer) 
{
       if(rc==TPF_SEND_MESSAGE_INV_EGROUP)
       {
           sprintf(message_buffer,MSGAPI001TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             001,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_NO_REQUEST)
       {
           sprintf(message_buffer,MSGAPI002TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             002,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_INV_TIMEOUT)
       {
           sprintf(message_buffer,MSGAPI003TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             003,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_REQ_SMALL)
       {
           sprintf(message_buffer,MSGAPI004TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             004,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_BUFF_SMALL)
       {
           sprintf(message_buffer,MSGAPI005TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             005,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_TIMEOUT)
       {
           sprintf(message_buffer,MSGAPI006TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             006,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_SOCK_ERR)
       {
           sprintf(message_buffer,MSGAPI007TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             007,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_RESP_SMALL)
       {
           sprintf(message_buffer,MSGAPI008TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             010,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
       else if(rc==TPF_SEND_MESSAGE_NO_SESSION)
       {
           sprintf(message_buffer,MSGAPI009TXT);
           conn_drvr_message(message_buffer,
                             WTOPC_NO_CHAIN,
                             DRVR_MSG_HEADER,
                             011,
                             'E',
                             COMMAND_CODE);
           exit(0);
       }
}

/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/* Name:            conn_drvr_message(char *msgbuf,                  */
/*                                    char msg_chain,                */
/*                                    char msg_header,               */
/*                                    int  msgnum,                   */
/*                                    char msglet,                   */
/*                                    char *msg_prefix)              */
/*                                                                   */
/*                  msgbuf     - ptr to msg text (null-terminated)   */
/*                                                                   */
/*                  msg_chain  - message chaining flag               */
/*                               WTOPC_NO_CHAIN                      */
/*                               WTOPC_CHAIN                         */
/*                               WTOPC_CHAIN_END                     */
/*                                                                   */
/*                  msg_header - 0 (no message header)               */
/*                             - nonzero (message header required)   */
/*                                                                   */
/*                  Following fields only valid if msg_header nonzero*/
/*                                                                   */
/*                  msgnum     - message number                      */
/*                  msglet     - message letter                      */
/*                  msg_prefix - message prefix (4-character id      */
/*                                 null-terminated)                  */
/*                                                                   */
/* Abstract:        Send conn driver message to terminal (determined */
/*                  by ECB ebrout field).                            */
/*                                                                   */
/* Description:     Send conn driver message to console.             */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* Interface Requirements:                                           */
/*                                                                   */
/* Input:                                                            */
/*                                                                   */
/*  Parameters:    None                                              */
/*                                                                   */
/*                                                                   */
/* Returns/Exits:  Returns                                           */
/*                                                                   */
/*  Function return value: None                                      */
/*                                                                   */
/*********************************************************************/
void conn_drvr_message(char *msgbuf, enum t_wtopc_chain msg_chain, char msg_header,
                       int msgnum, char msglet, char *msg_prefix) {

    struct wtopc_header  header_buffer;

    /*****************************************************************/
    /* Check if message needs to have a message header built.        */
    /*****************************************************************/
    if (msg_header != 0) {

       /**************************************************************/
       /* Build message header                                       */
       /**************************************************************/
       wtopc_insert_header(&header_buffer,
                           msg_prefix,
                           msgnum,
                           msglet,
                           WTOPC_SUBSYS_TIME);

       /**************************************************************/
       /* Issue message to terminal based on ECB ebrout.             */
       /**************************************************************/
       wtopc (msgbuf,WTOPC_EBROUT,msg_chain,&header_buffer);
       }
    /*****************************************************************/
    /* Issue message without message header.                         */
    /*****************************************************************/
    else
       wtopc (msgbuf,WTOPC_EBROUT,msg_chain,WTOPC_NO_HEADER);

    return;
}
