/**********************************************************************/
/*      Licensed Materials - Property of IBM                          */
/*      "Restricted Materials of IBM"                                 */
/*      5748-T15                                                      */
/*      Copyright IBM Corp. 2018                                      */
/*      All Rights Reserved                                           */
/*      US Government Users Restricted Rights - Use, duplication      */
/*      or disclosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                                */
/**********************************************************************/
/*                                                                    */
/* FUNCTION NAME.....QHSS                                             */
/* PURPOSE...........QHSS driver                                      */
/* FUNCTION TYPE.....TPF                                              */
/*                                                                    */
/* INPUT.......ZINET ACT                                              */
/*                                                                    */
/* Options:                                                           */
/*                                                                    */
/* DESCRIPTION: QHSS is the server program for testing the high       */
/*              speed connector. This header is also used for         */
/*              related drivers QHST and QHSU which are test the      */
/*              high speed connector with an SSL connection.          */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DEPENDENCIES... None.                                              */
/*                                                                    */
/* RESTRICTIONS... None.                                              */
/*                                                                    */
/* SPECIAL CONSIDERATIONS...                                          */
/*                                                                    */
/* INTERFACES:                                                        */
/*                                                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* CHANGE ACTIVITY...                                                 */
/*                                                                    */
/*   PLOG       DATE       DESCRIPTION                                */
/*   ----       ----       -----------                                */
/*              04/28/2016 Create High Speed Connector server driver  */
/*   d26322     08/24/2018 Added cleanup subroutine drivers using SSL */
/**********************************************************************/
#ifndef QHSS_H
#define QHSS_H

typedef struct {
    int response_delay;
    int response_required;
    int response_message_size;
} qhss_parm;

/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/* Name:            conn_drvr_message(char *msgbuf,                  */
/*                                    char msg_chain,                */
/*                                    char msg_header,               */
/*                                    int  msgnum,                   */
/*                                    char msglet,                   */
/*                                    char *msg_prefix)              */
/*                                                                   */
/*                  msgbuf     - ptr to msg text (null-terminated)   */
/*                                                                   */
/*                  msg_chain  - message chaining flag               */
/*                               WTOPC_NO_CHAIN                      */
/*                               WTOPC_CHAIN                         */
/*                               WTOPC_CHAIN_END                     */
/*                                                                   */
/*                  msg_header - 0 (no message header)               */
/*                             - nonzero (message header required)   */
/*                                                                   */
/*                  Following fields only valid if msg_header nonzero*/
/*                                                                   */
/*                  msgnum     - message number                      */
/*                  msglet     - message letter                      */
/*                  msg_prefix - message prefix (4-character id      */
/*                                 null-terminated)                  */
/*                                                                   */
/* Abstract:        Send conn driver message to terminal (determined */
/*                  by ECB ebrout field).                            */
/*                                                                   */
/* Description:     Send conn driver message to console.             */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* Interface Requirements:                                           */
/*                                                                   */
/* Input:                                                            */
/*                                                                   */
/*  Parameters:    None                                              */
/*                                                                   */
/*                                                                   */
/* Returns/Exits:  Returns                                           */
/*                                                                   */
/*  Function return value: None                                      */
/*                                                                   */
/*********************************************************************/
void conn_drvr_message(char *msgbuf, enum t_wtopc_chain msg_chain, char msg_header, //F25726
                       int msgnum, char msglet, char *msg_prefix);

void conn_drvr_message(char *msgbuf, enum t_wtopc_chain msg_chain, char msg_header, //F25726
                       int msgnum, char msglet, char *msg_prefix) {

    struct wtopc_header  header_buffer;

    /*****************************************************************/
    /* Check if message needs to have a message header built.        */
    /*****************************************************************/
    if (msg_header != 0) {

       /**************************************************************/
       /* Build message header                                       */
       /**************************************************************/
       wtopc_insert_header(&header_buffer,
                           msg_prefix,
                           msgnum,
                           msglet,
                           WTOPC_SUBSYS_TIME);

       /**************************************************************/
       /* Issue message to terminal based on ECB ebrout.             */
       /**************************************************************/
       wtopc (msgbuf,WTOPC_EBROUT,msg_chain,&header_buffer);
       }
    /*****************************************************************/
    /* Issue message without message header.                         */
    /*****************************************************************/
    else
       wtopc (msgbuf,WTOPC_EBROUT,msg_chain,WTOPC_NO_HEADER);

    return;
}

/*********************************************************************/
/*                                                                   */
/* Function:                                                         */
/*                                                                   */
/* Name:            SSL_cleanup(SSL *myssl, int conn_sock)           */
/*                                                                   */
/*                  myssl     - pointer to SSL token                 */
/*                                                                   */				
/*                  conn_sock - open socket file descriptor          */				
/*                                                                   */
/* Abstract:        Helper function for cleaning up SSL connections  */
/*                  before exiting.                                  */
/*                                                                   */
/* Description:     Cleans up SSL connection for driver using SSL    */
/*                  connections (ex: QHST, QHSU)                     */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* Interface Requirements:                                           */
/*                                                                   */
/* Input:                                                            */
/*                                                                   */
/*  Parameters:    SSL *myssl, int conn_sock                         */
/*                                                                   */
/*                                                                   */
/* Returns/Exits:  Returns                                           */
/*                                                                   */
/*  Function return value: int depending on success of close(socket) */
/*                         call                                      */
/*                                                                   */
/*********************************************************************/
#ifdef HEADER_SSL_H                                                             //d26322
static inline int SSL_cleanup(SSL *myssl, int conn_sock) {                      //d26322
    SSL_free(myssl);                                                            //d26322
	return close(conn_sock);                                                    //d26322
}                                                                               //d26322
#endif                                                                          //d26322

#endif
